@@ -49,6 +49,9 @@ typedef bool grn_bool;
 typedef __bf16 grn_bfloat16;
 #endif
 
+/**
+ * \brief Means "does not exist"
+ */
 #define GRN_ID_NIL (0x00)
 #define GRN_ID_MAX (0x3fffffff)
 
@@ -56,6 +59,7 @@ typedef __bf16 grn_bfloat16;
 #define GRN_FALSE  false
 
 typedef enum {
+  /// Success (0)
   GRN_SUCCESS = 0,
   GRN_END_OF_DATA = 1,
   GRN_UNKNOWN_ERROR = -1,
@@ -79,6 +83,7 @@ typedef enum {
   GRN_NO_SUCH_DEVICE = -19,
   GRN_NOT_A_DIRECTORY = -20,
   GRN_IS_A_DIRECTORY = -21,
+  /// Invalid function argument (-22)
   GRN_INVALID_ARGUMENT = -22,
   GRN_TOO_MANY_OPEN_FILES_IN_SYSTEM = -23,
   GRN_TOO_MANY_OPEN_FILES = -24,
@@ -92,6 +97,7 @@ typedef enum {
   GRN_DOMAIN_ERROR = -32,
   GRN_RESULT_TOO_LARGE = -33,
   GRN_RESOURCE_DEADLOCK_AVOIDED = -34,
+  /// Not enough memory available (-35)
   GRN_NO_MEMORY_AVAILABLE = -35,
   GRN_FILENAME_TOO_LONG = -36,
   GRN_NO_LOCKS_AVAILABLE = -37,
@@ -130,6 +136,7 @@ typedef enum {
   GRN_TOO_LARGE_OFFSET = -68,
   GRN_TOO_SMALL_LIMIT = -69,
   GRN_CAS_ERROR = -70,
+  /// When a version not supported by the default version set is specified (-71)
   GRN_UNSUPPORTED_COMMAND_VERSION = -71,
   GRN_NORMALIZER_ERROR = -72,
   GRN_TOKEN_FILTER_ERROR = -73,
@@ -152,12 +159,20 @@ GRN_API const char *
 grn_get_global_error_message(void);
 
 typedef enum {
+  /// The default encoding specified at build time (0).
+  /// The default at build time is "UTF-8".
   GRN_ENC_DEFAULT = 0,
+  /// Process strings as binary data (1)
   GRN_ENC_NONE,
+  /// EUC-JP (2)
   GRN_ENC_EUC_JP,
+  /// UTF-8 (3)
   GRN_ENC_UTF8,
+  /// Shift_JIS (4)
   GRN_ENC_SJIS,
+  /// Latin-1 (5)
   GRN_ENC_LATIN1,
+  /// KOI8-R (6)
   GRN_ENC_KOI8R
 } grn_encoding;
 
@@ -281,8 +296,19 @@ grn_ctx_pop_temporary_open_space(grn_ctx *ctx);
 GRN_API grn_rc
 grn_ctx_merge_temporary_open_space(grn_ctx *ctx);
 
+/**
+ * \return The default encoding
+ */
 GRN_API grn_encoding
 grn_get_default_encoding(void);
+/**
+ * \brief Set the default encoding
+ *
+ * \param encoding New encoding
+ *
+ * \return \ref GRN_SUCCESS on success, \ref GRN_INVALID_ARGUMENT on
+ *         error
+ */
 GRN_API grn_rc
 grn_set_default_encoding(grn_encoding encoding);
 
@@ -304,8 +330,19 @@ grn_get_package(void);
 GRN_API const char *
 grn_get_package_label(void);
 
+/**
+ * \return Default command version
+ */
 GRN_API grn_command_version
 grn_get_default_command_version(void);
+/**
+ * \brief Set default command version
+ *
+ * \param version New command version
+ *
+ * \return \ref GRN_SUCCESS on success, \ref GRN_UNSUPPORTED_COMMAND_VERSION on
+ *         error
+ */
 GRN_API grn_rc
 grn_set_default_command_version(grn_command_version version);
 GRN_API grn_command_version
@@ -361,9 +398,26 @@ GRN_API size_t
 grn_get_memory_map_size(void);
 
 /* grn_encoding */
-
+/**
+ * \brief Return string representation for the encoding. For example,
+ *        `grn_encoding_to_string(GRN_ENC_UTF8)` returns `"utf8"`.
+ *
+ * \param encoding The encoding
+ *
+ * \return String representation for the encoding on success, "unknown"
+ *         on invalid encoding.
+ */
 GRN_API const char *
 grn_encoding_to_string(grn_encoding encoding);
+/**
+ * \brief Parse encoding name and return \ref grn_encoding. For example,
+ *        `grn_encoding_parse("UTF8")` returns \ref GRN_ENC_UTF8.
+ *
+ * \param name The encoding name
+ *
+ * \return \ref grn_encoding matching `name` on success, \ref GRN_ENC_UTF8 on
+ *         invalid encoding name.
+ */
 GRN_API grn_encoding
 grn_encoding_parse(const char *name);
 
@@ -383,14 +437,32 @@ typedef uint32_t grn_column_flags;
 #define GRN_OBJ_TABLE_DAT_KEY   (0x02)
 #define GRN_OBJ_TABLE_NO_KEY    (0x03)
 
-#define GRN_OBJ_KEY_MASK        (0x07 << 3)
-#define GRN_OBJ_KEY_UINT        (0x00 << 3)
-#define GRN_OBJ_KEY_INT         (0x01 << 3)
-#define GRN_OBJ_KEY_FLOAT       (0x02 << 3)
-#define GRN_OBJ_KEY_GEO_POINT   (0x03 << 3)
-
-#define GRN_OBJ_KEY_WITH_SIS    (0x01 << 6)
-#define GRN_OBJ_KEY_NORMALIZE   (0x01 << 7)
+/// Mask of `GRN_OBJ_KEY_*`
+#define GRN_OBJ_KEY_MASK (0x07 << 3)
+/// Unsigned integer
+/// (Used to create data types. Used to determine sorting method.)
+#define GRN_OBJ_KEY_UINT (0x00 << 3)
+/// Signed integer
+/// (Used to create data types. Used to determine sorting method.)
+#define GRN_OBJ_KEY_INT (0x01 << 3)
+/// Float
+/// (Used to create data types. Used to determine sorting method.)
+#define GRN_OBJ_KEY_FLOAT (0x02 << 3)
+/// Latitude and longitude (\ref grn_geo_point)
+/// (Used to create data types. Used to determine sorting method.)
+#define GRN_OBJ_KEY_GEO_POINT (0x03 << 3)
+/// Enable semi-infinite string support.
+/// This is only available with \ref GRN_OBJ_TABLE_PAT_KEY.
+/// You can use efficient suffix search with this but it requires more
+/// storage/memory size. Because it generates many additional data for efficient
+/// suffix search implicitly.
+#define GRN_OBJ_KEY_WITH_SIS (0x01 << 6)
+/**
+ * \deprecated This was used when only NormalizerAuto was available.
+ *             Now you can specify a normalizer, so there is no need
+ *             to use it.
+ */
+#define GRN_OBJ_KEY_NORMALIZE (0x01 << 7)
 
 /* flags for grn_obj_flags and grn_column_flags */
 
@@ -426,9 +498,15 @@ typedef uint32_t grn_column_flags;
 
 /* Don't use (0x01<<12) because it's used internally. */
 
-#define GRN_OBJ_NO_SUBREC    (0x00 << 13)
-#define GRN_OBJ_WITH_SUBREC  (0x01 << 13)
+#define GRN_OBJ_NO_SUBREC   (0x00 << 13)
+#define GRN_OBJ_WITH_SUBREC (0x01 << 13)
 
+/// It shows that the key is variable size not fixed size.
+/// If you're using DB API such as \ref grn_table_create(), you don't need to
+/// use this.
+/// You can use type object such as `ShortText` instead.
+/// If you're using a low-level table such as \ref grn_hash and \ref grn_pat
+/// directly, you can use this to show that the table uses variable size key.
 #define GRN_OBJ_KEY_VAR_SIZE (0x01 << 14)
 
 #define GRN_OBJ_TEMPORARY    (0x00 << 15)
@@ -469,8 +547,16 @@ typedef uint32_t grn_column_flags;
 /* obj types */
 
 #define GRN_VOID (0x00)
+/**
+ * Auto-extendable buffer that can be used for storing a number and variable
+ * size string.
+ */
 #define GRN_BULK (0x02)
-#define GRN_PTR  (0x03)
+/**
+ * Buffer that has a `grn_obj *`. The hold `grn_obj *` can be closed
+ * automatically when this is closed by specifying \ref GRN_OBJ_OWN flag.
+ */
+#define GRN_PTR (0x03)
 /* vector of fixed size (uniform) data especially grn_id */
 #define GRN_UVECTOR                 (0x04)
 #define GRN_PVECTOR                 (0x05) /* vector of grn_obj* */
@@ -533,7 +619,15 @@ struct _grn_obj {
 
 #define GRN_OBJ_REFER    (0x01 << 0)
 #define GRN_OBJ_OUTPLACE (0x01 << 1)
-#define GRN_OBJ_OWN      (0x01 << 5)
+/**
+ * A flag to represent that \ref GRN_PTR or \ref GRN_PVECTOR owns associated
+ * \ref grn_obj. When this flag is set, the associated \ref grn_obj will be
+ * automatically closed using \ref grn_obj_close when owning \ref GRN_PTR or
+ * \ref GRN_PVECTOR is closed.
+ *
+ * You can use this flag only with \ref GRN_PTR_INIT.
+ */
+#define GRN_OBJ_OWN (0x01 << 5)
 
 #define GRN_OBJ_INIT(obj, obj_type, obj_flags, obj_domain)                     \
   do {                                                                         \
@@ -720,6 +814,37 @@ typedef enum {
   GRN_OP_ORDERED_NEAR_PHRASE_PRODUCT,
 } grn_operator;
 
+/**
+ * \brief Retrieve a column or an accessor from a specified table or accessor.
+ *
+ *        This function returns a column corresponding to the given name from
+ *        the specified table. If the name does not correspond to any column, it
+ *        returns NULL. If the name is an accessor string, it returns the
+ *        corresponding accessor. Accessor strings are dot-concatenated column
+ *        names. Column names that are started with `_` such as `_id` and `_key`
+ *        are pseudo column names. This function returns an accessor for a
+ *        pseudo column name.
+ *        See https://groonga.org/docs/reference/columns/pseudo.html for pseudo
+ *        column.
+ *
+ *        Column name examples: `name`, `age`
+ *
+ *        Pseudo column name examples: `_key`, `_score`, `_nsubrecs`
+ *
+ *        Accessor string examples: `tag.name`, `user.bookmarks.url`
+ *
+ *        If this function returns an accessor, you must call `grn_obj_unlink()`
+ *        with it when it's no longer needed. You can use
+ *        `grn_obj_is_accessor()` to detect whether it's an accessor or not.
+ *
+ * \param ctx The context object
+ * \param table The target table or accessor from which the column or accessor
+ *              is retrieved.
+ * \param name The name of the column or an accessor string.
+ * \param name_size The length of the `name` string.
+ *
+ * \return The column or accessor, or NULL if not found.
+ */
 GRN_API grn_obj *
 grn_obj_column(grn_ctx *ctx,
                grn_obj *table,
@@ -790,8 +915,46 @@ GRN_API grn_rc
 grn_obj_set_element_info(
   grn_ctx *ctx, grn_obj *obj, grn_id id, grn_info_type type, grn_obj *value);
 
+/**
+ * \brief Retrieve the value of the record corresponding to the given ID
+ *        in the specified object.
+ *
+ * \param ctx The context object
+ * \param obj The target object from which to retrieve the value
+ * \param id The ID of the target record
+ * \param value The buffer to store the retrieved value (must be prepared by the
+ *              caller)
+ *
+ * \return The value of the specified record in the object.
+ */
 GRN_API grn_obj *
 grn_obj_get_value(grn_ctx *ctx, grn_obj *obj, grn_id id, grn_obj *value);
+/**
+ * \brief Retrieve an array of fixed-size column values starting from a
+ *        specified record ID.
+ *
+ *        This function retrieves values from a fixed-size column (`obj`),
+ *        starting at the record ID given by `offset`. The retrieved values are
+ *        stored in the array pointed to by `values`, and the number of records
+ *        that can be retrieved is returned.
+ *
+ * \attention It is not guaranteed that all record IDs within the specified
+ *            range are valid. For tables where records may have been deleted,
+ *            you must use functions such as \ref grn_table_at to check the
+ *            existence of each record.
+ *
+ * \note If an error occurs and the return value is `-1`, check `ctx->rc` for
+ *       the specific error code (e.g., \ref GRN_NO_MEMORY_AVAILABLE,
+ *       \ref GRN_INVALID_ARGUMENT). Additional details might be available in
+ *       `ctx->errbuf`.
+ *
+ * \param ctx The context object
+ * \param obj The target fixed-size column
+ * \param offset The starting record ID for retrieving values
+ * \param values A pointer to an array where the values will be stored
+ *
+ * \return The number of records retrieved, or `-1` if an error occurred.
+ */
 GRN_API int
 grn_obj_get_values(grn_ctx *ctx, grn_obj *obj, grn_id offset, void **values);
 
@@ -807,35 +970,107 @@ grn_obj_get_values(grn_ctx *ctx, grn_obj *obj, grn_id offset, void **values);
   } while (0)
 
 #define GRN_OBJ_SET_MASK (0x07)
-#define GRN_OBJ_SET      (0x01)
-#define GRN_OBJ_INCR     (0x02)
-#define GRN_OBJ_DECR     (0x03)
-#define GRN_OBJ_APPEND   (0x04)
-#define GRN_OBJ_PREPEND  (0x05)
-#define GRN_OBJ_GET      (0x01 << 4)
-#define GRN_OBJ_COMPARE  (0x01 << 5)
-#define GRN_OBJ_LOCK     (0x01 << 6)
-#define GRN_OBJ_UNLOCK   (0x01 << 7)
+/**
+ * \brief Replace the value of the record/column with the specified value
+ */
+#define GRN_OBJ_SET (0x01)
+/**
+ * \brief Add the specified value to the record/column value
+ */
+#define GRN_OBJ_INCR (0x02)
+/**
+ * \brief Subtract the specified value from the record/column value
+ */
+#define GRN_OBJ_DECR (0x03)
+/**
+ * \brief Append the specified value to the column value.
+ */
+#define GRN_OBJ_APPEND (0x04)
+/**
+ * \brief Prepend the specified value to the column value.
+ */
+#define GRN_OBJ_PREPEND (0x05)
+#define GRN_OBJ_GET     (0x01 << 4)
+#define GRN_OBJ_COMPARE (0x01 << 5)
+#define GRN_OBJ_LOCK    (0x01 << 6)
+#define GRN_OBJ_UNLOCK  (0x01 << 7)
 
+/**
+ * \brief Update the value of a record identified by the given ID in the
+ *        specified object.
+ *
+ *        This function updates the value of the record specified by `id` in the
+ *        given object (`obj`). If the corresponding record does not exist, it
+ *        returns \ref GRN_INVALID_ARGUMENT.
+ *
+ * \param ctx The context object.
+ * \param obj The target object where the value will be updated.
+ * \param id The ID of the record to be updated.
+ * \param value The value to be stored in the record.
+ * \param flags Only one flag can be specified, depending on the type of the
+ *              object (`obj`).
+ *              - For a table and a scalar column, use either \ref GRN_OBJ_SET,
+ *                \ref GRN_OBJ_INCR, or \ref GRN_OBJ_DECR.
+ *              - For a vector column, use either \ref GRN_OBJ_SET, \ref
+ *                GRN_OBJ_INCR, \ref GRN_OBJ_DECR \ref GRN_OBJ_APPEND or
+ *                \ref GRN_OBJ_PREPEND.
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_INVALID_ARGUMENT is returned if the
+ *         record does not exist.
+ */
 GRN_API grn_rc
 grn_obj_set_value(
   grn_ctx *ctx, grn_obj *obj, grn_id id, grn_obj *value, int flags);
+
+#define GRN_OBJ_REMOVE_DEPENDENT (0x01 << 0)
+#define GRN_OBJ_REMOVE_ENSURE    (0x01 << 1)
+
+/**
+ * \brief Remove the specified object.
+ *
+ *        This function frees the specified object (`obj`) from memory. If the
+ *        object is a persistent object, it also removes the associated files
+ *        from disk.
+ *
+ * \param ctx The context object.
+ * \param obj The target object to be removed.
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ */
 GRN_API grn_rc
 grn_obj_remove(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc
 grn_obj_remove_dependent(grn_ctx *ctx, grn_obj *obj);
 GRN_API grn_rc
+grn_obj_remove_flags(grn_ctx *ctx, grn_obj *obj, uint32_t flags);
+/* Deprecated since 14.0.5. Use grn_ctx_remove(...,
+ * GRN_OBJ_REMOVE_ENSURE) instead. */
+GRN_API grn_rc
 grn_obj_remove_force(grn_ctx *ctx, const char *name, int name_size);
 GRN_API grn_rc
+grn_ctx_remove(grn_ctx *ctx, const char *name, int name_size, uint32_t flags);
+GRN_API grn_rc
+grn_ctx_remove_by_id(grn_ctx *ctx, grn_id id, uint32_t flags);
+
+/**
+ * \brief Rename a persistent object in the database used by the context.
+ *
+ *        This function updates the name of the specified object (`obj`) to the
+ *        new name provided. The object must be a persistent object.
+ *
+ * \param ctx The context object.
+ * \param obj The target object to be renamed.
+ * \param name The new name for the given object.
+ * \param name_size The size of the `name` in bytes.
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ */
+GRN_API grn_rc
 grn_obj_rename(grn_ctx *ctx,
                grn_obj *obj,
                const char *name,
                unsigned int name_size);
-GRN_API grn_rc
-grn_table_rename(grn_ctx *ctx,
-                 grn_obj *table,
-                 const char *name,
-                 unsigned int name_size);
 
 GRN_API grn_rc
 grn_column_rename(grn_ctx *ctx,
@@ -843,8 +1078,57 @@ grn_column_rename(grn_ctx *ctx,
                   const char *name,
                   unsigned int name_size);
 
+/**
+ * \brief Close an object.
+ *
+ *        This function frees all resources used by the specified object (`obj`)
+ *        from memory. All resources include other associated objects.
+ *
+ *        In general, you must close temporary objects explicitly. You don't
+ *        need to close persistent objects explicitly because you can close
+ *        persistent objects implicitly by closing a DB object.
+ *
+ *        You can use \ref grn_obj_unlink instead. It closes temporary objects
+ *        but does nothing for most persistent objects. It's useful for normal
+ *        use cases.
+ *
+ * \attention In general, you should not close persistent objects such as tables
+ *            and columns for performance reasons. If you close persistent
+ *            objects, you need to re-open them when they are needed again. This
+ *            is inefficient.
+ *
+ * \param ctx The context object.
+ * \param obj The object to be closed.
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ */
 GRN_API grn_rc
 grn_obj_close(grn_ctx *ctx, grn_obj *obj);
+/**
+ * \brief Reinitialize an object.
+ *
+ *        Buffer objects, \ref GRN_BULK, \ref GRN_PTR, \ref GRN_UVECTOR,
+ *        \ref GRN_PVECTOR, and \ref GRN_VECTOR, are only target objects
+ *        of this function. You can't use other objects such as table and
+ *        column for this function.
+ *
+ *        This function frees the current data in the specified object
+ *        (`obj`) and initializes the specified `obj` for the specified
+ *        `domain` and `flags`.
+ *
+ *        Before calling this function, the object must have been initialized.
+ *        You can use `GRN_XXX_INIT()` macros such as \ref GRN_TEXT_INIT to
+ *        initialize a buffer object.
+ *
+ * \param ctx The context object.
+ * \param obj The object to be reinitialized. It must be a buffer object.
+ * \param domain The new type that the object can hold.
+ * \param flags `0` or \ref GRN_OBJ_VECTOR. If \ref GRN_OBJ_VECTOR is
+ *              specified, the object will be configured to store a vector of
+ *              values of the specified `domain`.
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ */
 GRN_API grn_rc
 grn_obj_reinit(grn_ctx *ctx, grn_obj *obj, grn_id domain, uint8_t flags);
 /* On non reference count mode (default):
@@ -890,12 +1174,26 @@ grn_obj_unref_recursive(grn_ctx *ctx, grn_obj *obj);
 GRN_API void
 grn_obj_unref_recursive_dependent(grn_ctx *ctx, grn_obj *obj);
 
+/**
+ * \brief Returns a pointer to user data that can be registered in the object
+ *
+ * \param ctx The context object
+ * \param obj Only table, column, proc, and expr can be specified
+ * \return Pointer to user data
+ */
 GRN_API grn_user_data *
 grn_obj_user_data(grn_ctx *ctx, grn_obj *obj);
 
 GRN_API grn_rc
 grn_obj_set_finalizer(grn_ctx *ctx, grn_obj *obj, grn_proc_func *func);
-
+/**
+ * \brief Return the file path associated with an object.
+ *
+ * \param ctx The context object.
+ * \param obj The object whose file path is to be retrieved.
+ *
+ * \return The file path on success, `NULL` if the object is temporary.
+ */
 GRN_API const char *
 grn_obj_path(grn_ctx *ctx, grn_obj *obj);
 GRN_API int
@@ -1033,6 +1331,11 @@ typedef enum {
   GRN_HOOK_SELECT
 } grn_hook_entry;
 
+#define GRN_N_HOOK_ENTRIES 5 /* (GRN_HOOK_SELECT + 1) */
+
+GRN_API const char *
+grn_hook_entry_to_string(grn_hook_entry entry);
+
 GRN_API grn_rc
 grn_obj_add_hook(grn_ctx *ctx,
                  grn_obj *obj,
@@ -1572,9 +1875,10 @@ grn_ctx_recv_handler_set(grn_ctx *,
 /* various values exchanged via grn_obj */
 
 #define GRN_OBJ_DO_SHALLOW_COPY (GRN_OBJ_REFER | GRN_OBJ_OUTPLACE)
-#define GRN_OBJ_VECTOR          (0x01 << 7)
+/* A flag to initialize a vector object. */
+#define GRN_OBJ_VECTOR       (0x01 << 7)
 
-#define GRN_OBJ_MUTABLE(obj)    ((obj) && (obj)->header.type <= GRN_VECTOR)
+#define GRN_OBJ_MUTABLE(obj) ((obj) && (obj)->header.type <= GRN_VECTOR)
 
 #define GRN_VALUE_FIX_SIZE_INIT(obj, flags, domain)                            \
   GRN_OBJ_INIT((obj),                                                          \
