@@ -37,6 +37,14 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_temporary(grn_ctx *ctx, grn_obj *obj);
 
+/**
+ * \brief Check whether a Groonga built-in object is or not.
+ *
+ * \param ctx The context object
+ * \param obj The target object to check
+ *
+ * \return `true` if the object is a built-in Groonga object, `false` otherwise.
+ */
 GRN_API bool
 grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
@@ -87,6 +95,14 @@ GRN_API bool
 grn_obj_is_text_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
 grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj);
+/**
+ * \brief Check whether the specified object is a vector column or not.
+ *
+ * \param ctx The context object
+ * \param obj The target object to check
+ *
+ * \return `true` if the object is a vector column, `false` otherwise.
+ */
 GRN_API bool
 grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj);
 GRN_API bool
@@ -183,6 +199,10 @@ grn_obj_set_flag_to_string(int flags);
 
 GRN_API const char *
 grn_obj_type_to_string(uint8_t type);
+GRN_API bool
+grn_obj_type_is_table(uint8_t type);
+GRN_API bool
+grn_obj_type_is_column(uint8_t type);
 
 GRN_API grn_bool
 grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len);
