@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2009-2018  Brazil
-  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,34 @@ extern "C" {
 
 #define GRN_TABLE_MAX_KEY_SIZE (0x1000)
 
+/**
+ * \brief Define a new table in the DB used by ctx
+ *
+ * \param ctx The context object
+ * \param name Name of the table to create. If `NULL`, it is an unnamed table.
+ *             When creating a named table for a persistent db,
+ *             GRN_OBJ_PERSISTENT must be specified in `flags`
+ * \param name_size Length of `name`
+ * \param path File path of the table to create.
+ *             Enabled only if GRN_OBJ_PERSISTENT is specified in `flags`.
+ *             If `NULL`, the file path is automatically assigned.
+ * \param flags
+ *    * GRN_OBJ_TABLE_NO_KEY: Array table
+ *    * GRN_OBJ_TABLE_HASH_KEY: Hash table
+ *    * GRN_OBJ_TABLE_PAT_KEY: Patricia trie
+ *    * GRN_OBJ_KEY_WITH_SIS: All suffixes in the key string are automatically
+ *      registered
+ *    * GRN_OBJ_PERSISTENT: Persistent table
+ *    * GRN_OBJ_KEY_NORMALIZE: Normalized string is the key
+ * \param key_type Built-in type or table. If table B is created with table A
+ *                 as key_type, B will always be a subset of A.
+ *                 Disabled if GRN_OBJ_TABLE_NO_KEY is specified.
+ * \param value_type Type of the value for key.
+ *                   A table can have only one value for key, in addition to
+ *                   column
+ *
+ * \return A newly created table on success, `NULL` on error
+ */
 GRN_API grn_obj *
 grn_table_create(grn_ctx *ctx,
                  const char *name,
@@ -56,42 +84,164 @@ grn_table_create_similar(grn_ctx *ctx,
                                (flags),                                        \
                                (key_type),                                     \
                                (value_type))))
-
-/* TODO: int *added -> grn_bool *added */
+/**
+ * \brief Add a new record with `key` to the table and return its ID.
+ *        If `key` already exists in the table, returns the ID of the record.
+ *        For tables with GRN_OBJ_TABLE_NO_KEY, `key` and `key_size` are ignored
+ *
+ * \param ctx The context object
+ * \param table Target table
+ * \param key Search key
+ * \param key_size Length of `key`
+ * \param added If a non `NULL` value is specified, 1 is set when a new record
+ *              is added, and 0 is set when it is an existing record
+ *
+ * \return Record ID on success, GRN_ID_NIL on error
+ *
+ * \todo int *added -> bool *added
+ */
 GRN_API grn_id
 grn_table_add(grn_ctx *ctx,
               grn_obj *table,
               const void *key,
               unsigned int key_size,
               int *added);
+
+/**
+ * \brief It finds a record that has `key` parameter and returns ID of the found
+ *        record. If table parameter is a database, it finds an object (table,
+ *        column and so on) that has key parameter and returns ID of the found
+ *        object.
+ *
+ * \param ctx The context object
+ * \param table The table or database
+ * \param key The record or object key to be found
+ * \param key_size Length of `key`
+ *
+ * \return ID of the found object on success, \ref GRN_ID_NIL on not found or
+ *         error
+ */
 GRN_API grn_id
 grn_table_get(grn_ctx *ctx,
               grn_obj *table,
               const void *key,
               unsigned int key_size);
+/**
+ * \brief Search the table for a record of ID and return the specified ID if it
+ *        exists, or \ref GRN_ID_NIL if it does not
+ *
+ * \attention It is costly to perform. Do not call them frequently
+ *
+ * \param ctx The context object
+ * \param table The table or database
+ * \param id The ID to be found
+ *
+ * \return Specified ID on success, \ref GRN_ID_NIL on not found or error
+ */
 GRN_API grn_id
 grn_table_at(grn_ctx *ctx, grn_obj *table, grn_id id);
+/**
+ * \brief Execute longest common prefix (LCP) search and return ID found.
+ *
+ * \details Execute longest common prefix search if the table is
+ *          \ref GRN_TABLE_PAT_KEY or \ref GRN_TABLE_DAT_KEY.
+ *          If the table is \ref GRN_TABLE_HASH_KEY, search by exact match.
+ *
+ * \param ctx The context object.
+ * \param table The table.
+ * \param key Search key.
+ * \param key_size Length of `key`.
+ *
+ * \return ID on success, \ref GRN_ID_NIL on not found or error.
+ */
 GRN_API grn_id
 grn_table_lcp_search(grn_ctx *ctx,
                      grn_obj *table,
                      const void *key,
                      unsigned int key_size);
+/**
+ * \brief Get the key assigned to the ID.
+ *        If the size of the key found is larger than `buf_size`, it is not
+ *        stored in `keybuf`.
+ *
+ * \param ctx The context object.
+ * \param table The table.
+ * \param id The ID to be found.
+ * \param keybuf Buffer to store the record key.
+ * \param buf_size Size of `keybuf` in bytes.
+ *
+ * \return Key size of the record on success, `0` on not existed.
+ */
 GRN_API int
 grn_table_get_key(
   grn_ctx *ctx, grn_obj *table, grn_id id, void *keybuf, int buf_size);
+/**
+ * \brief Delete the record matching the key in the table.
+ *
+ * \param ctx The context object
+ * \param table The table
+ * \param key Key of record to be deleted
+ * \param key_size Size of `key` in bytes
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_INVALID_ARGUMENT is returned if the record
+ *         does not exist.
+ */
 GRN_API grn_rc
 grn_table_delete(grn_ctx *ctx,
                  grn_obj *table,
                  const void *key,
                  unsigned int key_size);
+/**
+ * \brief Delete the record matching the ID in the table.
+ *
+ * \param ctx The context object
+ * \param table The table
+ * \param id ID of record to be deleted
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_INVALID_ARGUMENT is returned if the record
+ *         does not exist.
+ */
 GRN_API grn_rc
 grn_table_delete_by_id(grn_ctx *ctx, grn_obj *table, grn_id id);
+/**
+ * \brief Update the key of the record matching ID in the table.
+ *        This operation supports only \ref GRN_TABLE_DAT_KEY.
+ *
+ * \param ctx The context object.
+ * \param table The \ref GRN_TABLE_DAT_KEY table.
+ * \param id ID of record to be updated.
+ * \param dest_key New key.
+ * \param dest_key_size Size of `dest_key` in bytes.
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_OPERATION_NOT_PERMITTED is returned if the
+ *         table other than \ref GRN_TABLE_DAT_KEY is specified.
+ */
 GRN_API grn_rc
 grn_table_update_by_id(grn_ctx *ctx,
                        grn_obj *table,
                        grn_id id,
                        const void *dest_key,
                        unsigned int dest_key_size);
+/**
+ * \brief Change the key of the record matching the `src_key` of the table.
+ *        Specify the new key and its byte length in `dest_key` and
+ *        `dest_key_size`. This operation is allowed only for table of
+ *        type GRN_TABLE_DAT_KEY
+ *
+ * \param ctx The context object
+ * \param table Target table
+ * \param src_key Key of record to be updated
+ * \param src_key_size Length of `src_key` (byte)
+ * \param dest_key New key
+ * \param dest_key_size Length of `dest_key_size` (byte)
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_OPERATION_NOT_PERMITTED is returned if not
+ *         \ref GRN_TABLE_DAT_KEY.
+ */
 GRN_API grn_rc
 grn_table_update(grn_ctx *ctx,
                  grn_obj *table,
@@ -99,6 +249,24 @@ grn_table_update(grn_ctx *ctx,
                  unsigned int src_key_size,
                  const void *dest_key,
                  unsigned int dest_key_size);
+/**
+ * \brief Delete all records in table.
+ *
+ * \attention Do not use in multi-threading. Because it might access data that
+ *            has already been deleted and crash when it does.
+ *
+ * \attention You need to reopen the target table and its columns in another
+ *            process when you use this in multi-processing. This removes
+ *            related files and creates new files. If you keep using the target
+ *            table and its columns in another process, the target table and
+ *            its columns in another process will touch removed data. It will
+ *            cause a crash or something wrong.
+ *
+ * \param ctx The context object
+ * \param table The table
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ */
 GRN_API grn_rc
 grn_table_truncate(grn_ctx *ctx, grn_obj *table);
 
@@ -114,6 +282,81 @@ grn_table_truncate(grn_ctx *ctx, grn_obj *table);
 #define GRN_CURSOR_SIZE_BY_BIT (0x01 << 5)
 #define GRN_CURSOR_RK          (0x01 << 6)
 
+/**
+ * \brief Creates and returns a cursor to retrieve the records registered in
+ *        the table in order
+ *
+ * \param ctx The context object
+ * \param table Target table
+ * \param min Minimum limit of key (`NULL` means no minimum limit).
+ *            See below for GRN_CURSOR_PREFIX
+ * \param min_size Size of min. See below for GRN_CURSOR_PREFIX
+ * \param max Maximum limit of key (`NULL` means no maximum limit).
+ *            See below for GRN_CURSOR_PREFIX
+ * \param max_size Size of max. GRN_CURSOR_PREFIX may be ignored
+ * \param offset Extracts records from the range of records that meet the
+ *               condition, starting with the `offset`-th (offset is
+ *               zero-based). When GRN_CURSOR_PREFIX is specified, negative
+ *               numbers cannot be specified
+ * \param limit Only `limit` records in the range that meet the condition are
+ *              extracted. -1 means all.
+ *              When GRN_CURSOR_PREFIX is specified, negative numbers cannot
+ *              be specified
+ * \param flags
+ *     * GRN_CURSOR_ASCENDING: Retrieve records in ascending order
+ *       * If GRN_CURSOR_PREFIX is specified and a record with a near key is
+ *         retrieved, or a common prefix search is used, it will be ignored
+ *     * GRN_CURSOR_DESCENDING: Retrieve records in descending order
+ *       * If GRN_CURSOR_PREFIX is specified and a record with a near key is
+ *         retrieved, or a common prefix search is used, it will be ignored
+ *     * GRN_CURSOR_GT: a key that matches `min` is not included in the cursor
+ *       range
+ *       * If `min` is `NULL`, or if GRN_CURSOR_PREFIX is specified and a record
+ *         with a near key is retrieved, or a common prefix search is used,
+ *         it will be ignored
+ *     * GRN_CURSOR_LT: a key that matches `max` is not included in the cursor
+ *       range
+ *       * If `max` is `NULL`, or if GRN_CURSOR_PREFIX is specified, it will
+ *         be ignored
+ *     * GRN_CURSOR_BY_ID: Retrieves records in ID order
+ *       * If GRN_CURSOR_PREFIX is specified, it will be ignored
+ *     * GRN_CURSOR_BY_KEY: Retrieves records in key order
+ *       * It can be used with table that specify GRN_OBJ_TABLE_PAT_KEY
+ *       * For table with GRN_OBJ_TABLE_HASH_KEY or GRN_OBJ_TABLE_NO_KEY, it
+ *         will be ignored
+ *     * GRN_CURSOR_PREFIX: A cursor is created to retrieve the following
+ *       records for the table with GRN_OBJ_TABLE_PAT_KEY
+ *       * If `max` is `NULL`, retrieve the record for which key is a prefix
+ *         match to `min`. `max_size` is ignored
+ *       * If `max` and `max_size` are specified and the table key is of type
+ *         `ShortText`, then a `max` and common prefix search is executed and
+ *         records with a common prefix
+ *         greater than or equal to min_size bytes are retrieved. `min` is
+ *         ignored
+ *       * If `max` and `max_size` are specified and the key of the table is a
+ *         fixed-length type, records are retrieved sequentially from nodes that
+ *         are near each other on the `max` and PAT tree.
+ *         * But, records are not retrieved for nodes in the PAT tree of key for
+ *           bits less than `min_size` bytes and corresponding to nodes on a
+ *           different branch than `max`.
+ *           Being near a position on the PAT tree is not the same as being near
+ *           a key value.
+ *           In this case, `max` must be as wide as or greater than the key size
+ *           of the target table. `min` is ignored
+ *     * GRN_CURSOR_BY_ID, GRN_CURSOR_BY_KEY and GRN_CURSOR_PREFIX cannot be
+ *       specified at the same time.
+ *     * In a table created with GRN_OBJ_TABLE_PAT_KEY, if GRN_CURSOR_PREFIX and
+ *       GRN_CURSOR_RK are specified, retrieves records where key is a prefix
+ *       matching a string of lower case alphabetic characters converted to
+ *       half-width kana characters according to JIS X 4063:2000 (this standard
+ *       is abolished).
+ *       * Supports only GRN_ENC_UTF8
+ *       * GRN_CURSOR_ASCENDING and GRN_CURSOR_DESCENDING are invalid.
+ *         Records cannot be retrieved in ascending or descending order of key
+ *         value
+ *
+ * \return  A newly opened table cursor on success, `NULL` on error
+ */
 GRN_API grn_table_cursor *
 grn_table_cursor_open(grn_ctx *ctx,
                       grn_obj *table,
@@ -124,12 +367,52 @@ grn_table_cursor_open(grn_ctx *ctx,
                       int offset,
                       int limit,
                       int flags);
+/**
+ * \brief Free the cursor created by grn_table_cursor_open()
+ *
+ * \param ctx The context object
+ * \param tc Target cursor
+ *
+ * \return \ref GRN_SUCCESS on success, GRN_INVALID_ARGUMENT if `tc` is invalid
+ */
 GRN_API grn_rc
 grn_table_cursor_close(grn_ctx *ctx, grn_table_cursor *tc);
+
+/**
+ * \brief Move the cursur forward to the next and return its record ID.
+ *        Return GRN_ID_NIL when the end is reached
+ *
+ * \param ctx The context object
+ * \param tc Target cursor
+ *
+ * \return Record ID on there is a next, GRN_ID_NIL on no next or on error
+ */
 GRN_API grn_id
 grn_table_cursor_next(grn_ctx *ctx, grn_table_cursor *tc);
+
+/**
+ * \brief Sets the key of the current record in cursor to `key` parameter and
+ *        returns its length
+ *
+ * \param ctx The context object
+ * \param tc Target cursor
+ * \param key A pointer to the key of the current record is set
+ *
+ * \return Length of `key`
+ */
 GRN_API int
 grn_table_cursor_get_key(grn_ctx *ctx, grn_table_cursor *tc, void **key);
+
+/**
+ * \brief Sets the value of the current record in cursor to `value` parameter
+ *        and returns its length
+ *
+ * \param ctx The context object
+ * \param tc Target cursor
+ * \param value A pointer to the value of the current record is set
+ *
+ * \return Length of `value`
+ */
 GRN_API int
 grn_table_cursor_get_value(grn_ctx *ctx, grn_table_cursor *tc, void **value);
 GRN_API uint32_t
@@ -138,15 +421,48 @@ grn_table_cursor_get_key_value(grn_ctx *ctx,
                                void **key,
                                uint32_t *key_size,
                                void **value);
+/**
+ * \brief Set `value` to the current record according to `flags`
+ *
+ * \param ctx The context object
+ * \param tc Table cursor
+ * \param value Value to set
+ * \param flags Available values:
+ *      * \ref GRN_OBJ_SET
+ *      * \ref GRN_OBJ_INCR
+ *      * \ref GRN_OBJ_DECR
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_INVALID_ARGUMENT is returned if the current
+ *         record for cursor does not exist.
+ */
 GRN_API grn_rc
 grn_table_cursor_set_value(grn_ctx *ctx,
                            grn_table_cursor *tc,
                            const void *value,
                            int flags);
+/**
+ * \brief Delete the current record for cursor
+ *
+ * \param ctx The context object
+ * \param tc Target cursor
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_INVALID_ARGUMENT is returned if the current
+ *         record for cursor does not exist.
+ */
 GRN_API grn_rc
 grn_table_cursor_delete(grn_ctx *ctx, grn_table_cursor *tc);
 GRN_API size_t
 grn_table_cursor_get_max_n_records(grn_ctx *ctx, grn_table_cursor *cursor);
+/**
+ * \brief Return the table of cursor
+ *
+ * \param ctx The context object
+ * \param tc Target cursor
+ *
+ * \return The table of cursor on success, `NULL` on error
+ */
 GRN_API grn_obj *
 grn_table_cursor_table(grn_ctx *ctx, grn_table_cursor *tc);
 
@@ -214,7 +530,7 @@ grn_table_cursor_foreach(grn_ctx *ctx,
     } else {                                                                   \
       (ctx)->seqno++;                                                          \
     }                                                                          \
-    if (table) {                                                               \
+    if ((table) && grn_table_size(ctx, (table)) > 0) {                         \
       switch ((table)->header.type) {                                          \
       case GRN_TABLE_PAT_KEY:                                                  \
         GRN_PAT_EACH((ctx),                                                    \
@@ -262,7 +578,7 @@ grn_table_cursor_foreach(grn_ctx *ctx,
 
 #define GRN_TABLE_EACH_BEGIN(ctx, table, cursor, id)                           \
   do {                                                                         \
-    if ((table)) {                                                             \
+    if ((table) && grn_table_size(ctx, (table)) > 0) {                         \
       grn_table_cursor *cursor;                                                \
       cursor = grn_table_cursor_open((ctx),                                    \
                                      (table),                                  \
@@ -279,7 +595,7 @@ grn_table_cursor_foreach(grn_ctx *ctx,
 
 #define GRN_TABLE_EACH_BEGIN_FLAGS(ctx, table, cursor, id, flags)              \
   do {                                                                         \
-    if ((table)) {                                                             \
+    if ((table) && grn_table_size(ctx, (table)) > 0) {                         \
       grn_table_cursor *cursor;                                                \
       cursor = grn_table_cursor_open((ctx),                                    \
                                      (table),                                  \
@@ -296,7 +612,7 @@ grn_table_cursor_foreach(grn_ctx *ctx,
 
 #define GRN_TABLE_EACH_BEGIN_MIN(ctx, table, cursor, id, min, min_size, flags) \
   do {                                                                         \
-    if ((table)) {                                                             \
+    if ((table) && grn_table_size(ctx, (table)) > 0) {                         \
       grn_table_cursor *cursor;                                                \
       cursor = grn_table_cursor_open((ctx),                                    \
                                      (table),                                  \
@@ -322,7 +638,13 @@ grn_table_cursor_foreach(grn_ctx *ctx,
 typedef struct _grn_table_sort_key grn_table_sort_key;
 typedef unsigned char grn_table_sort_flags;
 
-#define GRN_TABLE_SORT_ASC  (0x00 << 0)
+/**
+ * \brief Sort in ascending order
+ */
+#define GRN_TABLE_SORT_ASC (0x00 << 0)
+/**
+ * \brief Sort in descending order
+ */
 #define GRN_TABLE_SORT_DESC (0x01 << 0)
 
 struct _grn_table_sort_key {
@@ -331,6 +653,26 @@ struct _grn_table_sort_key {
   int offset;
 };
 
+/**
+ * \brief Sort the records in table and store the results in `result`
+ *
+ * \param ctx The context object
+ * \param table Target table
+ * \param offset Starting offset of the sorted record.
+ *               The records are stored in order from the offset (zero-based).
+ * \param limit Maximum number of records to be stored in `result`
+ * \param result Table to store results
+ * \param keys Array of sort keys
+ *             * keys.key: You can specify either column of table, accessor of
+ *               table, or proc
+ *             * keys.flags: You can specify the next
+ *               * \ref GRN_TABLE_SORT_ASC
+ *               * \ref GRN_TABLE_SORT_DESC
+ *             * keys.offset: Member for internal use
+ * \param n_keys Number of elements in `keys` array
+ *
+ * \return Number of sorted records
+ */
 GRN_API int
 grn_table_sort(grn_ctx *ctx,
                grn_obj *table,
@@ -420,15 +762,58 @@ grn_table_group_keys_parse(grn_ctx *ctx,
                            const char *raw_sort_keys,
                            int32_t raw_sort_keys_size,
                            uint32_t *n_keys);
+/**
+ * \brief Execute the set operation of `table1` and `table2` according to
+ *        the specification of `op` and store the result into `res`.
+ *
+ * \attention The table specified in `res` will be destructed.
+ *
+ * \param ctx The context object.
+ * \param table1 The table.
+ * \param table2 The table.
+ * \param res Specify `table1` or `table2`.
+ * \param op Type of operation.
+ *           - \ref GRN_OP_OR
+ *           - \ref GRN_OP_AND
+ *           - \ref GRN_OP_AND_NOT
+ *           - \ref GRN_OP_ADJUST
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error.
+ *         For example, \ref GRN_INVALID_ARGUMENT is returned if `table1` is
+ *         `NULL`.
+ */
 GRN_API grn_rc
 grn_table_setoperation(grn_ctx *ctx,
                        grn_obj *table1,
                        grn_obj *table2,
                        grn_obj *res,
                        grn_operator op);
+/**
+ * \brief Delete duplicated records in `table1` and `table2`.
+ *
+ * \param ctx The context object.
+ * \param table1 The table.
+ * \param table2 The table.
+ * \param res1 Specify `table1`.
+ * \param res2 Specify `table2`.
+ *
+ * \return \ref GRN_SUCCESS on success, \ref GRN_INVALID_ARGUMENT on
+ *         `table1 != res1 || table2 != res2`.
+ */
 GRN_API grn_rc
 grn_table_difference(
   grn_ctx *ctx, grn_obj *table1, grn_obj *table2, grn_obj *res1, grn_obj *res2);
+/**
+ * \brief Store the column IDs of the table starting with `name` in `res`.
+ *
+ * \param ctx The context object.
+ * \param table The table.
+ * \param name The column name prefix.
+ * \param name_size Size of `name` in bytes. If `0`, you get all column IDs.
+ * \param res The \ref GRN_TABLE_HASH_KEY table to store results.
+ *
+ * \return The number of IDs.
+ */
 GRN_API int
 grn_table_columns(grn_ctx *ctx,
                   grn_obj *table,
@@ -436,9 +821,29 @@ grn_table_columns(grn_ctx *ctx,
                   unsigned int name_size,
                   grn_obj *res);
 
+/**
+ * \brief Return the number of records registered in the table
+ *
+ * \param ctx The context object
+ * \param table The table or database
+ *
+ * \return The number of records
+ */
 GRN_API unsigned int
 grn_table_size(grn_ctx *ctx, grn_obj *table);
 
+/**
+ *
+ * \brief Rename table to `name`. All columns of the table are renamed at the
+ *        same time. The table must be a persistent object.
+ *
+ * \param ctx The context object
+ * \param table Target table
+ * \param name New name
+ * \param name_size Size of `name` in bytes
+ *
+ * \return \ref GRN_SUCCESS on success, the appropriate \ref grn_rc on error
+ */
 GRN_API grn_rc
 grn_table_rename(grn_ctx *ctx,
                  grn_obj *table,
@@ -469,7 +874,7 @@ grn_table_sort_key_close(grn_ctx *ctx,
                          grn_table_sort_key *keys,
                          uint32_t nkeys);
 
-GRN_API grn_bool
+GRN_API bool
 grn_table_is_grouped(grn_ctx *ctx, grn_obj *table);
 
 GRN_API unsigned int
@@ -498,7 +903,7 @@ grn_table_tokenize(grn_ctx *ctx,
                    const char *str,
                    unsigned int str_len,
                    grn_obj *buf,
-                   grn_bool addp);
+                   bool addp);
 
 GRN_API grn_rc
 grn_table_apply_expr(grn_ctx *ctx,
@@ -519,6 +924,9 @@ grn_table_get_duplicated_keys(grn_ctx *ctx,
 GRN_API bool
 grn_table_have_duplicated_keys(grn_ctx *ctx, grn_obj *table);
 
+GRN_API bool
+grn_table_have_tokenizer(grn_ctx *ctx, grn_obj *table);
+
 typedef struct _grn_table_selector grn_table_selector;
 
 GRN_API grn_table_selector *
@@ -561,6 +969,12 @@ grn_table_selector_get_max_n_enough_filtered_records(
 GRN_API grn_rc
 grn_table_selector_set_max_n_enough_filtered_records(
   grn_ctx *ctx, grn_table_selector *table_selector, int64_t n);
+GRN_API bool
+grn_table_selector_get_ensure_using_select_result(
+  grn_ctx *ctx, grn_table_selector *table_selector);
+GRN_API grn_rc
+grn_table_selector_set_ensure_using_select_result(
+  grn_ctx *ctx, grn_table_selector *table_selector, bool use);
 GRN_API uint32_t
 grn_table_selector_get_fuzzy_max_distance(grn_ctx *ctx,
                                           grn_table_selector *table_selector);
