@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2009-2018  Brazil
+  Copyright (C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -608,25 +608,32 @@ grn_table_group_add_subrec(grn_ctx *ctx,
 
 typedef struct {
   grn_obj *object;
+  grn_obj *accessor;
   grn_obj *range;
   bool is_expr;
   grn_expr_executor expr_executor;
   bool is_reference_column;
+  bool is_vector;
 } group_key;
 
 static grn_inline void
 group_key_init(grn_ctx *ctx,
                group_key *key,
-               grn_obj *object)
+               grn_obj *object,
+               grn_obj *accessor)
 {
   key->object = object;
-  grn_id range_id = grn_obj_get_range(ctx, key->object);
+  key->accessor = accessor;
+  grn_id range_id;
+  grn_obj_flags range_flags = 0;
+  grn_obj_get_range_info(ctx, key->object, &range_id, &range_flags);
   key->range = grn_ctx_at(ctx, range_id);
   key->is_expr = grn_obj_is_expr(ctx, key->object);
   if (key->is_expr) {
     grn_expr_executor_init(ctx, &(key->expr_executor), key->object);
   }
   key->is_reference_column = grn_obj_is_table(ctx, key->range);
+  key->is_vector = (range_flags & GRN_OBJ_VECTOR) ? true : false;
 }
 
 static grn_inline void
@@ -654,16 +661,137 @@ group_key_get_value(grn_ctx *ctx,
   }
 }
 
+typedef struct {
+  grn_obj *table;
+  group_key *keys;
+  int n_keys;
+  grn_table_group_result *results;
+  int n_results;
+  unsigned int n_records;
+  int max_n_target_records;
+  int nth_record;
+  grn_obj bulk;
+  grn_obj vector;
+  grn_obj vector_pack_header;
+  grn_obj vector_pack_footer;
+  grn_id id;
+  grn_rset_recinfo *ri;
+} multi_keys_data;
+
+static grn_inline void
+grn_table_group_multi_keys_add_record(grn_ctx *ctx, multi_keys_data *data)
+{
+  grn_obj *bulk = &(data->bulk);
+  grn_obj *vector = &(data->vector);
+  grn_obj *vector_pack_header = &(data->vector_pack_header);
+  grn_obj *vector_pack_footer = &(data->vector_pack_footer);
+
+  int i;
+  for (i = 0; i < data->n_results; i++) {
+    grn_table_group_result *result = data->results + i;
+
+    if (data->nth_record > result->limit) {
+      break;
+    }
+
+    uint32_t offset;
+    uint32_t n;
+    if (data->n_keys == 1) {
+      offset = 0;
+      n = grn_vector_size(ctx, vector);
+    } else {
+      offset = result->key_begin;
+      int end;
+      if (result->key_end > data->n_keys) {
+        end = data->n_keys;
+      } else {
+        end = result->key_end + 1;
+      }
+      n = (uint32_t)(end - result->key_begin);
+    }
+    GRN_BULK_REWIND(vector_pack_header);
+    GRN_BULK_REWIND(vector_pack_footer);
+    grn_obj *body = grn_vector_pack(ctx,
+                                    vector,
+                                    offset,
+                                    n,
+                                    0,
+                                    vector_pack_header,
+                                    vector_pack_footer);
+    GRN_BULK_REWIND(bulk);
+#ifdef GRN_WITH_XXHASH
+    bool need_hash_value = false;
+    if (grn_table_group_all_hash_value_enable) {
+      need_hash_value = true;
+    } else if (grn_table_group_fallback_hash_value_enable) {
+      const size_t total_size =
+        GRN_TEXT_LEN(vector_pack_header) +
+        GRN_TEXT_LEN(body) +
+        GRN_TEXT_LEN(vector_pack_footer);
+      if (total_size > GRN_TABLE_MAX_KEY_SIZE) {
+        need_hash_value = true;
+      }
+    }
+#endif
+    GRN_TEXT_PUT(ctx,
+                 bulk,
+                 GRN_TEXT_VALUE(vector_pack_header),
+                 GRN_TEXT_LEN(vector_pack_header));
+    GRN_TEXT_PUT(ctx, bulk, GRN_TEXT_VALUE(body), GRN_TEXT_LEN(body));
+    GRN_TEXT_PUT(ctx,
+                 bulk,
+                 GRN_TEXT_VALUE(vector_pack_footer),
+                 GRN_TEXT_LEN(vector_pack_footer));
+#ifdef GRN_WITH_XXHASH
+    if (need_hash_value) {
+      XXH128_hash_t hash_value = XXH3_128bits(GRN_TEXT_VALUE(bulk),
+                                              GRN_TEXT_LEN(bulk));
+      GRN_BULK_REWIND(bulk);
+      /* This value must not be conflicted with the first byte of data
+       * packed by grn_vector_pack(). The first byte is generated by
+       * grn_text_benc(ctx, header, n) where n is the number of
+       * drilldown keys. n must be less than 0x1020408f. See
+       * GRN_B_ENC() for 0x1020408f. */
+      const uint8_t hash_value_mark = 0x8f;
+      GRN_UINT8_PUT(ctx, bulk, hash_value_mark);
+      GRN_UINT64_PUT(ctx, bulk, hash_value.low64);
+      GRN_UINT64_PUT(ctx, bulk, hash_value.high64);
+    }
+#endif
+
+    void *value;
+    grn_id group_id = grn_table_add_v(ctx,
+                                      result->table,
+                                      GRN_BULK_HEAD(bulk),
+                                      (int)GRN_BULK_VSIZE(bulk),
+                                      &value,
+                                      NULL);
+    if (group_id != GRN_ID_NIL) {
+      grn_table_group_add_subrec(ctx,
+                                 result->table,
+                                 group_id,
+                                 value,
+                                 data->ri ? data->ri->score : 0,
+                                 (grn_rset_posinfo *)&(data->id),
+                                 0,
+                                 bulk);
+    }
+  }
+}
+
 typedef struct {
   grn_obj bulk;
   grn_obj value_buffer;
-  bool need_resolved_id;
+  int resolve_id_depth;
   grn_obj *table;
   group_key key;
   grn_obj *range;
   grn_ra_cache cache;
   grn_obj *res;
+  grn_table_flags flags;
   bool with_subrec;
+  /* for GRN_TABLE_GROUP_KEY_VECTOR_EXPANSION_* */
+  multi_keys_data multi_keys_data;
 } single_key_records_data;
 
 static grn_inline grn_id
@@ -672,15 +800,86 @@ grn_table_group_single_key_records_resolve_id(grn_ctx *ctx,
                                               grn_table_cursor *cursor,
                                               grn_id id)
 {
-  if (data->need_resolved_id) {
+  if (data->resolve_id_depth == 1) {
     void *key;
     grn_table_cursor_get_key(ctx, cursor, &key);
     return *((grn_id *)key);
+  } else if (data->resolve_id_depth > 1) {
+    return grn_accessor_resolve_id(ctx, data->key.accessor, id);
   } else {
     return id;
   }
 }
 
+typedef struct {
+  multi_keys_data *multi_keys_data;
+  grn_obj *vector;
+  bool is_uvector;
+  uint32_t element_size;
+  uint32_t n_elements;
+} power_set_data;
+
+static void
+grn_table_group_key_vector_expand_power_set_internal(grn_ctx *ctx,
+                                                     power_set_data *data,
+                                                     uint32_t i)
+{
+  grn_obj *key = &(data->multi_keys_data->vector);
+  if (data->is_uvector) {
+    uint32_t offset = (data->element_size * i);
+    grn_vector_add_element(ctx,
+                           key,
+                           GRN_BULK_HEAD(data->vector) + offset,
+                           data->element_size,
+                           0,
+                           data->vector->header.domain);
+  } else {
+    const char *content;
+    unsigned int content_length;
+    grn_id domain;
+    content_length = grn_vector_get_element(ctx,
+                                            data->vector,
+                                            i,
+                                            &content,
+                                            NULL,
+                                            &domain);
+    grn_vector_add_element(ctx, key, content, content_length, 0, domain);
+  }
+  grn_table_group_multi_keys_add_record(ctx, data->multi_keys_data);
+  if (i == data->n_elements - 1) {
+    return;
+  }
+  uint32_t j;
+  for (j = i + 1; j < data->n_elements; j++) {
+    grn_table_group_key_vector_expand_power_set_internal(ctx, data, j);
+    const char *content;
+    grn_vector_pop_element(ctx, key, &content, NULL, NULL);
+  }
+  return;
+}
+
+static void
+grn_table_group_key_vector_expand_power_set(grn_ctx *ctx,
+                                            multi_keys_data *multi_keys_data,
+                                            grn_obj *vector)
+{
+  power_set_data data;
+  data.multi_keys_data = multi_keys_data;
+  data.vector = vector;
+  data.is_uvector = grn_obj_is_uvector(ctx, vector);
+  if (data.is_uvector) {
+    data.element_size = grn_uvector_element_size(ctx, vector);
+  } else {
+    data.element_size = 0;
+  }
+  data.n_elements = grn_vector_size(ctx, vector);
+  uint32_t i;
+  for (i = 0; i < data.n_elements; i++) {
+    GRN_BULK_REWIND(&(multi_keys_data->vector));
+    grn_table_group_key_vector_expand_power_set_internal(ctx, &data, i);
+  }
+}
+
 static grn_rc
 grn_table_group_single_key_records_foreach_fix_size(grn_ctx *ctx,
                                                     grn_table_cursor *cursor,
@@ -755,11 +954,19 @@ grn_table_group_single_key_records_foreach_var_size_reference(
     grn_table_group_single_key_records_resolve_id(ctx, data, cursor, id);
   grn_io_win jw;
   uint32_t value_size;
-  grn_id *references = grn_ja_ref(ctx, ja, value_id, &jw, &value_size);
-  uint32_t n_references = value_size / sizeof(grn_id);
+  uint8_t *value = grn_ja_ref(ctx, ja, value_id, &jw, &value_size);
+  uint32_t element_size;
+  bool with_weight = grn_obj_is_weight_vector_column(ctx, data->key.object);
+  if (with_weight) {
+    element_size = sizeof(grn_id) + sizeof(float);
+  } else {
+    element_size = sizeof(grn_id);
+  }
+  uint32_t n_references = value_size / element_size;
   uint32_t i;
   for (i = 0; i < n_references; i++) {
-    grn_id reference = references[i];
+    uint8_t *element = value + (element_size * i);
+    grn_id reference = *((grn_id *)element);
     if (reference == GRN_ID_NIL) {
       continue;
     }
@@ -810,6 +1017,7 @@ grn_table_group_single_key_records_foreach(grn_ctx *ctx,
 
   if (data->with_subrec) {
     grn_table_cursor_get_value(ctx, cursor, (void **)&ri);
+    data->multi_keys_data.ri = ri;
   }
   grn_id value_id =
     grn_table_group_single_key_records_resolve_id(ctx, data, cursor, id);
@@ -817,7 +1025,12 @@ grn_table_group_single_key_records_foreach(grn_ctx *ctx,
     group_key_get_value(ctx, &(data->key), value_id, &(data->bulk));
   switch (bulk->header.type) {
   case GRN_UVECTOR :
-    {
+    if (data->flags & GRN_TABLE_GROUP_KEY_VECTOR_EXPANSION_POWER_SET) {
+      grn_table_group_key_vector_expand_power_set(ctx,
+                                                  &(data->multi_keys_data),
+                                                  bulk);
+      data->multi_keys_data.n_records++;
+    } else {
       unsigned int element_size;
       uint8_t *elements;
       size_t i, n_elements;
@@ -854,14 +1067,23 @@ grn_table_group_single_key_records_foreach(grn_ctx *ctx,
     }
     break;
   case GRN_VECTOR :
-    {
+    if (data->flags & GRN_TABLE_GROUP_KEY_VECTOR_EXPANSION_POWER_SET) {
+      grn_table_group_key_vector_expand_power_set(ctx,
+                                                  &(data->multi_keys_data),
+                                                  bulk);
+      data->multi_keys_data.n_records++;
+    } else {
       unsigned int i, n_elements;
       n_elements = grn_vector_size(ctx, bulk);
       for (i = 0; i < n_elements; i++) {
         const char *content;
         unsigned int content_length;
-        content_length = grn_vector_get_element(ctx, bulk, i,
-                                                &content, NULL, NULL);
+        content_length = grn_vector_get_element(ctx,
+                                                bulk,
+                                                i,
+                                                &content,
+                                                NULL,
+                                                NULL);
         grn_id group_id = grn_table_add_v(ctx,
                                           data->res,
                                           content,
@@ -912,26 +1134,32 @@ grn_table_group_single_key_records(grn_ctx *ctx, grn_obj *table,
   GRN_VOID_INIT(&(data.value_buffer));
   data.table = table;
   data.res = result->table;
-  int limit = -1;
-  if (result->flags & GRN_TABLE_GROUP_LIMIT) {
-    limit = result->limit;
-  }
+  data.flags = result->flags;
   if ((cursor = grn_table_cursor_open(ctx, table,
                                       NULL, 0,
                                       NULL, 0,
-                                      0, limit, 0))) {
+                                      0, result->limit, 0))) {
     grn_obj *real_key = key;
-    data.need_resolved_id = false;
+    grn_obj *accessor = NULL;
+    data.resolve_id_depth = 0;
     if (key->header.type == GRN_ACCESSOR) {
-      grn_accessor *a = (grn_accessor *)key;
-      if (a->action == GRN_ACCESSOR_GET_KEY &&
-          a->next && a->next->action == GRN_ACCESSOR_GET_COLUMN_VALUE &&
-          a->next->obj && !a->next->next) {
-        real_key = a->next->obj;
-        data.need_resolved_id = true;
+      grn_accessor *a;
+      int depth = 0;
+      accessor = key;
+      for (a = (grn_accessor *)accessor;
+           a->action == GRN_ACCESSOR_GET_KEY && a->next;
+           a = a->next, depth++) {
+        /* do nothing */
+      }
+      if (a &&
+          a->action == GRN_ACCESSOR_GET_COLUMN_VALUE &&
+          a->obj &&
+          !a->next) {
+        real_key = a->obj;
+        data.resolve_id_depth = depth;
       }
     }
-    group_key_init(ctx, &(data.key), real_key);
+    group_key_init(ctx, &(data.key), real_key, accessor);
     data.with_subrec = (DB_OBJ(table)->header.flags & GRN_OBJ_WITH_SUBREC);
     if (data.key.object->header.type == GRN_COLUMN_FIX_SIZE) {
       GRN_RA_CACHE_INIT((grn_ra *)(data.key.object), &(data.cache));
@@ -942,17 +1170,37 @@ grn_table_group_single_key_records(grn_ctx *ctx, grn_obj *table,
         &data);
       GRN_RA_CACHE_FIN(ctx, (grn_ra *)(data.key.object), &(data.cache));
     } else if (data.key.object->header.type == GRN_COLUMN_VAR_SIZE &&
-               data.key.is_reference_column) {
+               data.key.is_reference_column &&
+               !(data.flags & GRN_TABLE_GROUP_KEY_VECTOR_EXPANSION_POWER_SET)) {
       grn_table_cursor_foreach(
         ctx,
         cursor,
         grn_table_group_single_key_records_foreach_var_size_reference,
         &data);
     } else {
+      data.multi_keys_data.table = data.table;
+      data.multi_keys_data.keys = &(data.key);
+      data.multi_keys_data.n_keys = 1;
+      data.multi_keys_data.results = result;
+      data.multi_keys_data.n_results = 1;
+      data.multi_keys_data.n_records = grn_table_size(ctx, data.table);
+      data.multi_keys_data.max_n_target_records =
+        (int)(data.multi_keys_data.n_records);
+      data.multi_keys_data.nth_record = 0;
+      GRN_TEXT_INIT(&(data.multi_keys_data.bulk), 0);
+      GRN_OBJ_INIT(&(data.multi_keys_data.vector), GRN_VECTOR, 0, GRN_DB_VOID);
+      GRN_TEXT_INIT(&(data.multi_keys_data.vector_pack_header), 0);
+      GRN_TEXT_INIT(&(data.multi_keys_data.vector_pack_footer), 0);
+      data.multi_keys_data.id = GRN_ID_NIL;
+      data.multi_keys_data.ri = NULL;
       grn_table_cursor_foreach(ctx,
                                cursor,
                                grn_table_group_single_key_records_foreach,
                                &data);
+      GRN_OBJ_FIN(ctx, &(data.multi_keys_data.vector_pack_footer));
+      GRN_OBJ_FIN(ctx, &(data.multi_keys_data.vector_pack_header));
+      GRN_OBJ_FIN(ctx, &(data.multi_keys_data.vector));
+      GRN_OBJ_FIN(ctx, &(data.multi_keys_data.bulk));
     }
     group_key_fin(ctx, &(data.key));
     grn_table_cursor_close(ctx, cursor);
@@ -973,14 +1221,10 @@ grn_table_group_all_records(grn_ctx *ctx, grn_obj *table,
   grn_obj *res = result->table;
 
   GRN_VOID_INIT(&value_buffer);
-  int limit = -1;
-  if (result->flags & GRN_TABLE_GROUP_LIMIT) {
-    limit = result->limit;
-  }
   if ((tc = grn_table_cursor_open(ctx, table,
                                   NULL, 0,
                                   NULL, 0,
-                                  0, limit, 0))) {
+                                  0, result->limit, 0))) {
     void *value;
     grn_id group_id;
     if ((group_id = grn_table_add_v(ctx, res,
@@ -1114,119 +1358,8 @@ grn_table_group_with_range_gap(grn_ctx *ctx, grn_obj *table,
   return 0;
 }
 
-typedef struct {
-  grn_obj *table;
-  group_key *keys;
-  int n_keys;
-  grn_table_group_result *results;
-  int n_results;
-  unsigned int n_records;
-  int max_n_target_records;
-  int nth_record;
-  grn_obj bulk;
-  grn_obj vector;
-  grn_obj vector_pack_header;
-  grn_obj vector_pack_footer;
-  grn_id id;
-  grn_rset_recinfo *ri;
-} grn_table_group_multi_keys_data;
-
-static grn_inline void
-grn_table_group_multi_keys_add_record(grn_ctx *ctx,
-                                      grn_table_group_multi_keys_data *data)
-{
-  grn_obj *bulk = &(data->bulk);
-  grn_obj *vector = &(data->vector);
-  grn_obj *vector_pack_header = &(data->vector_pack_header);
-  grn_obj *vector_pack_footer = &(data->vector_pack_footer);
-
-  int i;
-  for (i = 0; i < data->n_results; i++) {
-    grn_table_group_result *result = data->results + i;
-
-    if (data->nth_record > result->limit) {
-      break;
-    }
-
-    int end;
-    if (result->key_end > data->n_keys) {
-      end = data->n_keys;
-    } else {
-      end = result->key_end + 1;
-    }
-    GRN_BULK_REWIND(vector_pack_header);
-    GRN_BULK_REWIND(vector_pack_footer);
-    grn_obj *body = grn_vector_pack(ctx,
-                                    vector,
-                                    result->key_begin,
-                                    (uint32_t)(end - result->key_begin),
-                                    0,
-                                    vector_pack_header,
-                                    vector_pack_footer);
-    GRN_BULK_REWIND(bulk);
-#ifdef GRN_WITH_XXHASH
-    bool need_hash_value = false;
-    if (grn_table_group_all_hash_value_enable) {
-      need_hash_value = true;
-    } else if (grn_table_group_fallback_hash_value_enable) {
-      const size_t total_size =
-        GRN_TEXT_LEN(vector_pack_header) +
-        GRN_TEXT_LEN(body) +
-        GRN_TEXT_LEN(vector_pack_footer);
-      if (total_size > GRN_TABLE_MAX_KEY_SIZE) {
-        need_hash_value = true;
-      }
-    }
-#endif
-    GRN_TEXT_PUT(ctx,
-                 bulk,
-                 GRN_TEXT_VALUE(vector_pack_header),
-                 GRN_TEXT_LEN(vector_pack_header));
-    GRN_TEXT_PUT(ctx, bulk, GRN_TEXT_VALUE(body), GRN_TEXT_LEN(body));
-    GRN_TEXT_PUT(ctx,
-                 bulk,
-                 GRN_TEXT_VALUE(vector_pack_footer),
-                 GRN_TEXT_LEN(vector_pack_footer));
-#ifdef GRN_WITH_XXHASH
-    if (need_hash_value) {
-      XXH128_hash_t hash_value = XXH3_128bits(GRN_TEXT_VALUE(bulk),
-                                              GRN_TEXT_LEN(bulk));
-      GRN_BULK_REWIND(bulk);
-      /* This value must not be conflicted with the first byte of data
-       * packed by grn_vector_pack(). The first byte is generated by
-       * grn_text_benc(ctx, header, n) where n is the number of
-       * drilldown keys. n must be less than 0x1020408f. See
-       * GRN_B_ENC() for 0x1020408f. */
-      const uint8_t hash_value_mark = 0x8f;
-      GRN_UINT8_PUT(ctx, bulk, hash_value_mark);
-      GRN_UINT64_PUT(ctx, bulk, hash_value.low64);
-      GRN_UINT64_PUT(ctx, bulk, hash_value.high64);
-    }
-#endif
-
-    void *value;
-    grn_id group_id = grn_table_add_v(ctx,
-                                      result->table,
-                                      GRN_BULK_HEAD(bulk),
-                                      (int)GRN_BULK_VSIZE(bulk),
-                                      &value,
-                                      NULL);
-    if (group_id != GRN_ID_NIL) {
-      grn_table_group_add_subrec(ctx,
-                                 result->table,
-                                 group_id,
-                                 value,
-                                 data->ri ? data->ri->score : 0,
-                                 (grn_rset_posinfo *)&(data->id),
-                                 0,
-                                 bulk);
-    }
-  }
-}
-
 static void
-grn_table_group_multi_keys_scalar_records(grn_ctx *ctx,
-                                          grn_table_group_multi_keys_data *data)
+grn_table_group_multi_keys_scalar_records(grn_ctx *ctx, multi_keys_data *data)
 {
   grn_table_cursor *tc = grn_table_cursor_open(ctx,
                                                data->table,
@@ -1267,7 +1400,7 @@ grn_table_group_multi_keys_scalar_records(grn_ctx *ctx,
 
 static grn_inline void
 grn_table_group_multi_keys_vector_record(grn_ctx *ctx,
-                                         grn_table_group_multi_keys_data *data,
+                                         multi_keys_data *data,
                                          grn_obj *key_buffers,
                                          int nth_key)
 {
@@ -1351,8 +1484,7 @@ grn_table_group_multi_keys_vector_record(grn_ctx *ctx,
 }
 
 static void
-grn_table_group_multi_keys_vector_records(grn_ctx *ctx,
-                                          grn_table_group_multi_keys_data *data)
+grn_table_group_multi_keys_vector_records(grn_ctx *ctx, multi_keys_data *data)
 {
   grn_table_cursor *tc = grn_table_cursor_open(ctx,
                                                data->table,
@@ -1431,7 +1563,20 @@ grn_table_group_results_prepare(grn_ctx *ctx,
 {
   int i;
   grn_table_group_result *rp;
+  uint32_t n_records = grn_table_size(ctx, table);
   for (i = 0, rp = results; i < n_results; i++, rp++) {
+    if (results[i].flags & GRN_TABLE_GROUP_LIMIT) {
+      int limit = results[i].limit;
+      if (limit < 0) {
+        limit += (int)(n_records + 1);
+        if (limit < 0) {
+          limit = 0;
+        }
+      }
+      results[i].limit = limit;
+    } else {
+      results[i].limit = (int)n_records;
+    }
     if (!rp->table) {
       grn_table_flags flags;
       grn_obj *key_type = NULL;
@@ -1443,19 +1588,26 @@ grn_table_group_results_prepare(grn_ctx *ctx,
       if (n_keys == 0 && n_results == 1) {
         key_type = grn_ctx_at(ctx, GRN_DB_SHORT_TEXT);
       } else if (n_keys == 1) {
-        grn_id key_type_id = grn_obj_get_range(ctx, keys[0].key);
-        if (key_type_id == GRN_ID_NIL) {
-          grn_obj inspected;
-          GRN_TEXT_INIT(&inspected, 0);
-          grn_inspect(ctx, &inspected, keys[0].key);
-          ERR(GRN_INVALID_ARGUMENT,
-              "[table][group] unknown key type: %.*s",
-              (int)GRN_TEXT_LEN(&inspected),
-              GRN_TEXT_VALUE(&inspected));
-          GRN_OBJ_FIN(ctx, &inspected);
-          return ctx->rc;
+        grn_id key_type_id;
+        grn_obj_flags key_type_flags = 0;
+        grn_obj_get_range_info(ctx, keys[0].key, &key_type_id, &key_type_flags);
+        if ((key_type_flags & GRN_OBJ_VECTOR) &&
+            rp->flags & GRN_TABLE_GROUP_KEY_VECTOR_EXPANSION_POWER_SET) {
+          flags |= GRN_OBJ_KEY_VAR_SIZE;
+        } else {
+          if (key_type_id == GRN_ID_NIL) {
+            grn_obj inspected;
+            GRN_TEXT_INIT(&inspected, 0);
+            grn_inspect(ctx, &inspected, keys[0].key);
+            ERR(GRN_INVALID_ARGUMENT,
+                "[table][group] unknown key type: %.*s",
+                (int)GRN_TEXT_LEN(&inspected),
+                GRN_TEXT_VALUE(&inspected));
+            GRN_OBJ_FIN(ctx, &inspected);
+            return ctx->rc;
+          }
+          key_type = grn_ctx_at(ctx, key_type_id);
         }
-        key_type = grn_ctx_at(ctx, key_type_id);
       } else {
         flags |= GRN_OBJ_KEY_VAR_SIZE;
       }
@@ -1564,7 +1716,7 @@ grn_table_group(grn_ctx *ctx, grn_obj *table,
     } else if (n_keys == 1 && n_results == 1) {
       grn_table_group_single_key_records(ctx, table, keys->key, results);
     } else {
-      grn_bool have_vector = GRN_FALSE;
+      bool have_vector = false;
       for (k = 0, kp = keys; k < n_keys; k++, kp++) {
         grn_id range_id;
         grn_obj_flags range_flags = 0;
@@ -1581,11 +1733,11 @@ grn_table_group(grn_ctx *ctx, grn_obj *table,
           goto exit;
         }
         if (range_flags & GRN_OBJ_VECTOR) {
-          have_vector = GRN_TRUE;
+          have_vector = true;
           break;
         }
       }
-      grn_table_group_multi_keys_data data;
+      multi_keys_data data;
       data.table = table;
       data.keys = GRN_MALLOCN(group_key, (size_t)n_keys);
       if (!data.keys) {
@@ -1597,27 +1749,15 @@ grn_table_group(grn_ctx *ctx, grn_obj *table,
       data.n_keys = n_keys;
       int i;
       for (i = 0; i < n_keys; i++) {
-        group_key_init(ctx, data.keys + i, keys[i].key);
+        group_key_init(ctx, data.keys + i, keys[i].key, NULL);
       }
       data.results = results;
       data.n_results = n_results;
       data.n_records = grn_table_size(ctx, table);
       data.max_n_target_records = -1;
       for (i = 0; i < n_results; i++) {
-        if (results[i].flags & GRN_TABLE_GROUP_LIMIT) {
-          int limit = results[i].limit;
-          if (limit < 0) {
-            limit += (int)(data.n_records + 1);
-            if (limit < 0) {
-              limit = 0;
-            }
-          }
-          results[i].limit = limit;
-          if (limit > data.max_n_target_records) {
-            data.max_n_target_records = limit;
-          }
-        } else {
-          results[i].limit = (int)(data.n_records);
+        if (results[i].limit > data.max_n_target_records) {
+          data.max_n_target_records = results[i].limit;
         }
       }
       if (data.max_n_target_records == -1) {
@@ -1655,7 +1795,7 @@ grn_table_group_keys_parse(grn_ctx *ctx,
                            grn_obj *table,
                            const char *raw_keys,
                            int32_t raw_keys_length,
-                           int *n_keys)
+                           uint32_t *n_keys)
 {
   return
     grn_table_sort_keys_parse_internal(ctx,
