@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2010-2016  Brazil
-  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2010-2016  Brazil
+  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -51,14 +51,15 @@ grn_nfkc_normalize_options_init(grn_ctx *ctx,
   options->char_type_func = char_type_func;
   options->decompose_func = decompose_func;
   options->compose_func = compose_func;
-  options->include_removed_source_location = GRN_TRUE;
-  options->report_source_offset = GRN_FALSE;
-  options->unify_kana = GRN_FALSE;
-  options->unify_kana_case = GRN_FALSE;
-  options->unify_kana_voiced_sound_mark = GRN_FALSE;
-  options->unify_hyphen = GRN_FALSE;
+  options->include_removed_source_location = true;
+  options->report_source_offset = false;
+  options->unify_latin_alphabet_with = false;
+  options->unify_kana = false;
+  options->unify_kana_case = false;
+  options->unify_kana_voiced_sound_mark = false;
+  options->unify_hyphen = false;
   options->unify_prolonged_sound_mark = GRN_FALSE;
-  options->unify_hyphen_and_prolonged_sound_mark = GRN_FALSE;
+  options->unify_hyphen_and_prolonged_sound_mark = false;
   options->unify_middle_dot = GRN_FALSE;
   options->unify_katakana_v_sounds = GRN_FALSE;
   options->unify_katakana_bu_sound = GRN_FALSE;
@@ -68,12 +69,14 @@ grn_nfkc_normalize_options_init(grn_ctx *ctx,
   options->unify_katakana_wo_sound = GRN_FALSE;
   options->unify_katakana_di_sound = GRN_FALSE;
   options->unify_katakana_gu_small_sounds = GRN_FALSE;
-  options->unify_kana_hyphen = GRN_FALSE;
-  options->unify_kana_prolonged_sound_mark = GRN_FALSE;
-  options->unify_katakana_trailing_o = GRN_FALSE;
-  options->unify_to_romaji = GRN_FALSE;
-  options->unify_to_katakana = GRN_FALSE;
+  options->unify_kana_hyphen = false;
+  options->unify_kana_prolonged_sound_mark = false;
+  options->unify_katakana_trailing_o = false;
+  options->unify_to_romaji = false;
+  options->unify_to_katakana = false;
   options->remove_blank = GRN_FALSE;
+  options->remove_blank_force = false;
+  options->remove_blank_force_is_set = false;
   options->remove_new_line = true;
   options->remove_symbol = false;
   options->strip = false;
@@ -146,6 +149,13 @@ grn_nfkc_normalize_options_apply(grn_ctx *ctx,
                                     raw_options,
                                     i,
                                     options->report_source_offset);
+    } else if (GRN_RAW_STRING_EQUAL_CSTRING(name_raw,
+                                            "unify_latin_alphabet_with")) {
+      options->unify_latin_alphabet_with =
+        grn_vector_get_element_bool(ctx,
+                                    raw_options,
+                                    i,
+                                    options->unify_latin_alphabet_with);
     } else if (GRN_RAW_STRING_EQUAL_CSTRING(name_raw, "unify_kana")) {
       options->unify_kana = grn_vector_get_element_bool(ctx,
                                                         raw_options,
@@ -273,6 +283,13 @@ grn_nfkc_normalize_options_apply(grn_ctx *ctx,
                                     raw_options,
                                     i,
                                     options->remove_blank);
+    } else if (GRN_RAW_STRING_EQUAL_CSTRING(name_raw, "remove_blank_force")) {
+      options->remove_blank_force =
+        grn_vector_get_element_bool(ctx,
+                                    raw_options,
+                                    i,
+                                    options->remove_blank_force);
+      options->remove_blank_force_is_set = true;
     } else if (GRN_RAW_STRING_EQUAL_CSTRING(name_raw, "remove_new_line")) {
       options->remove_new_line =
         grn_vector_get_element_bool(ctx,
