@@ -1,6 +1,6 @@
 /*
   Copyright (C) 2015-2018  Brazil
-  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2018-2024  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -74,6 +74,14 @@ grn_obj_is_true(grn_ctx *ctx, grn_obj *obj)
       return GRN_INT32_VALUE(obj) != 0;
     case GRN_DB_UINT32:
       return GRN_UINT32_VALUE(obj) != 0;
+#ifdef GRN_HAVE_BFLOAT16
+    case GRN_DB_BFLOAT16:
+      {
+        grn_bfloat16 bfloat16_value;
+        bfloat16_value = GRN_BFLOAT16_VALUE(obj);
+        return grn_bfloat16_is_zero(bfloat16_value);
+      }
+#endif
     case GRN_DB_FLOAT32:
       {
         float float_value;
@@ -137,49 +145,59 @@ grn_obj_is_temporary(grn_ctx *ctx, grn_obj *obj)
   return false;
 }
 
-grn_bool
+bool
 grn_obj_is_builtin(grn_ctx *ctx, grn_obj *obj)
 {
   grn_id id;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   id = grn_obj_id(ctx, obj);
   return grn_id_is_builtin(ctx, id);
 }
 
-grn_bool
+bool
 grn_obj_is_bulk(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_BULK;
 }
 
-grn_bool
+bool
 grn_obj_is_text_family_bulk(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_bulk(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return GRN_TYPE_IS_TEXT_FAMILY(obj->header.domain);
 }
 
-grn_bool
+bool
 grn_obj_is_number_family_bulk(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_bulk(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return grn_type_id_is_number_family(ctx, obj->header.domain);
 }
 
+bool
+grn_obj_maybe_record_bulk(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_bulk(ctx, obj)) {
+    return false;
+  }
+
+  return grn_id_maybe_table(ctx, obj->header.domain);
+}
+
 bool
 grn_obj_is_vector(grn_ctx *ctx, grn_obj *obj)
 {
@@ -194,12 +212,22 @@ bool
 grn_obj_is_text_family_vector(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_vector(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return grn_type_id_is_text_family(ctx, obj->header.domain);
 }
 
+bool
+grn_obj_is_number_family_vector(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_uvector(ctx, obj)) {
+    return false;
+  }
+
+  return grn_type_id_is_number_family(ctx, obj->header.domain);
+}
+
 bool
 grn_obj_is_weight_vector(grn_ctx *ctx, grn_obj *obj)
 {
@@ -240,13 +268,13 @@ grn_obj_is_db(grn_ctx *ctx, grn_obj *obj)
   return obj->header.type == GRN_DB;
 }
 
-grn_bool
+bool
 grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_table = GRN_FALSE;
+  bool is_table = false;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   switch (obj->header.type) {
@@ -254,7 +282,7 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
   case GRN_TABLE_HASH_KEY:
   case GRN_TABLE_PAT_KEY:
   case GRN_TABLE_DAT_KEY:
-    is_table = GRN_TRUE;
+    is_table = true;
     break;
   default:
     break;
@@ -266,7 +294,7 @@ grn_obj_is_table(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_table_with_key = false;
+  bool is_table_with_key = false;
 
   if (!obj) {
     return false;
@@ -287,7 +315,7 @@ grn_obj_is_table_with_key(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_table_with_value(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_table_with_value = false;
+  bool is_table_with_value = false;
 
   if (!obj) {
     return false;
@@ -425,20 +453,20 @@ grn_obj_is_result_set(grn_ctx *ctx, grn_obj *obj)
   return flags & GRN_OBJ_WITH_SUBREC;
 }
 
-grn_bool
+bool
 grn_obj_is_column(grn_ctx *ctx, grn_obj *obj)
 {
-  grn_bool is_column = GRN_FALSE;
+  bool is_column = false;
 
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   switch (obj->header.type) {
   case GRN_COLUMN_FIX_SIZE:
   case GRN_COLUMN_VAR_SIZE:
   case GRN_COLUMN_INDEX:
-    is_column = GRN_TRUE;
+    is_column = true;
   default:
     break;
   }
@@ -456,11 +484,11 @@ grn_obj_is_number_family_column(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_number_family(ctx, grn_obj_get_range(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_scalar_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return (obj->header.flags & GRN_OBJ_COLUMN_TYPE_MASK) ==
@@ -487,11 +515,11 @@ grn_obj_is_number_family_scalar_column(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_number_family(ctx, grn_obj_get_range(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_vector_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return (
@@ -509,11 +537,11 @@ grn_obj_is_text_family_vector_column(grn_ctx *ctx, grn_obj *obj)
   return grn_type_id_is_text_family(ctx, grn_obj_get_range(ctx, obj));
 }
 
-grn_bool
+bool
 grn_obj_is_weight_vector_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_vector_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return (obj->header.flags & GRN_OBJ_WITH_WEIGHT) == GRN_OBJ_WITH_WEIGHT;
@@ -523,17 +551,17 @@ bool
 grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   const grn_id range_id = grn_obj_get_range(ctx, obj);
   if (grn_id_is_builtin(ctx, range_id)) {
-    return GRN_FALSE;
+    return false;
   }
 
   grn_obj *range = range = grn_ctx_at(ctx, range_id);
   if (!range) {
-    return GRN_FALSE;
+    return false;
   }
 
   uint8_t range_type = range->header.type;
@@ -550,66 +578,74 @@ grn_obj_is_reference_column(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
-grn_bool
+bool
 grn_obj_is_data_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_COLUMN_FIX_SIZE ||
          obj->header.type == GRN_COLUMN_VAR_SIZE;
 }
 
-grn_bool
+/**
+ * \brief Check whether the specified object is an index column or not.
+ *
+ * \param ctx The context object
+ * \param obj The target object to check
+ *
+ * \return `true` if the object is exactly an index column, `false` otherwise.
+ */
+bool
 grn_obj_is_index_column(grn_ctx *ctx, grn_obj *obj)
 {
   if (!grn_obj_is_column(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_COLUMN_INDEX;
 }
 
-grn_bool
+bool
 grn_obj_is_accessor(grn_ctx *ctx, grn_obj *obj)
 {
   if (!obj) {
-    return GRN_FALSE;
+    return false;
   }
 
   return obj->header.type == GRN_ACCESSOR;
 }
 
-grn_bool
+bool
 grn_obj_is_id_accessor(grn_ctx *ctx, grn_obj *obj)
 {
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_ID;
 }
 
-grn_bool
+bool
 grn_obj_is_key_accessor(grn_ctx *ctx, grn_obj *obj)
 {
   grn_accessor *accessor;
 
   if (!grn_obj_is_accessor(ctx, obj)) {
-    return GRN_FALSE;
+    return false;
   }
 
   accessor = (grn_accessor *)obj;
   if (accessor->next) {
-    return GRN_FALSE;
+    return false;
   }
 
   return accessor->action == GRN_ACCESSOR_GET_KEY;
@@ -793,6 +829,38 @@ grn_obj_is_column_value_accessor(grn_ctx *ctx, grn_obj *obj)
   return accessor->action == GRN_ACCESSOR_GET_COLUMN_VALUE;
 }
 
+bool
+grn_obj_is_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_ID:
+  case GRN_ACCESSOR_GET_SCORE:
+  case GRN_ACCESSOR_GET_NSUBRECS:
+  case GRN_ACCESSOR_GET_MAX:
+  case GRN_ACCESSOR_GET_MIN:
+  case GRN_ACCESSOR_GET_SUM:
+  case GRN_ACCESSOR_GET_AVG:
+  case GRN_ACCESSOR_GET_MEAN:
+  case GRN_ACCESSOR_GET_VALUE:
+    return true;
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_scalar_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
 bool
 grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -827,6 +895,28 @@ grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
+bool
+grn_obj_is_vector_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_vector_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
 grn_bool
 grn_obj_is_type(grn_ctx *ctx, grn_obj *obj)
 {
@@ -909,6 +999,17 @@ grn_obj_is_selector_only_proc(grn_ctx *ctx, grn_obj *obj)
   return proc->funcs[PROC_INIT] == NULL;
 }
 
+bool
+grn_obj_is_applier_proc(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!grn_obj_is_function_proc(ctx, obj)) {
+    return false;
+  }
+
+  grn_proc *proc = (grn_proc *)obj;
+  return proc->callbacks.function.applier != NULL;
+}
+
 grn_bool
 grn_obj_is_normalizer_proc(grn_ctx *ctx, grn_obj *obj)
 {
@@ -1280,6 +1381,33 @@ grn_obj_type_to_string(uint8_t type)
   }
 }
 
+bool
+grn_obj_type_is_table(uint8_t type)
+{
+  switch (type) {
+  case GRN_TABLE_HASH_KEY:
+  case GRN_TABLE_PAT_KEY:
+  case GRN_TABLE_DAT_KEY:
+  case GRN_TABLE_NO_KEY:
+    return true;
+  default:
+    return false;
+  }
+}
+
+bool
+grn_obj_type_is_column(uint8_t type)
+{
+  switch (type) {
+  case GRN_COLUMN_FIX_SIZE:
+  case GRN_COLUMN_VAR_SIZE:
+  case GRN_COLUMN_INDEX:
+    return true;
+  default:
+    return false;
+  }
+}
+
 grn_bool
 grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len)
 {
@@ -1294,6 +1422,18 @@ grn_obj_name_is_column(grn_ctx *ctx, const char *name, int name_len)
   return memchr(name, GRN_DB_DELIMITER, (size_t)name_len) != NULL;
 }
 
+bool
+grn_obj_is_empty(grn_ctx *ctx, grn_obj *obj, grn_id id)
+{
+  if (obj->header.type == GRN_COLUMN_VAR_SIZE) {
+    return grn_ja_is_empty(ctx, (grn_ja *)obj, id);
+  }
+  ERR(GRN_FUNCTION_NOT_IMPLEMENTED,
+      "[obj][is-empty] unsupported type: %s",
+      grn_obj_type_to_string(obj->header.type));
+  return false;
+}
+
 grn_io *
 grn_obj_get_io(grn_ctx *ctx, grn_obj *obj)
 {
